#!/usr/bin/env python
#
# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the replication slave administration utility. It is used to
perform replication operations on one or more slaves.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import logging
import os.path
import signal
import sys

from mysql.utilities.exception import UtilError, UtilRplError
from mysql.utilities.common.messages import SCRIPT_THRESHOLD_WARNING
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.options import add_failover_options, add_rpl_user
from mysql.utilities.common.options import check_server_lists
from mysql.utilities.common.options import UtilitiesParser
from mysql.utilities.common.server import check_hostname_alias
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.common.topology import parse_failover_connections
from mysql.utilities.command.rpl_admin import RplCommands, purge_log
from mysql.utilities import VERSION_FRM

# Constants
NAME = "MySQL Utilities - mysqlfailover "
DESCRIPTION = "mysqlfailover - automatic replication health monitoring and failover"
USAGE = "%prog --master=root@localhost --discover-slaves-login=root " + \
        "--candidates=root@host123:3306,root@host456:3306 "
_DATE_FORMAT = '%Y-%m-%d %H:%M:%S %p'
_DATE_LEN = 22

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup a terminal signal handler for SIGNIT
# Must use SetConsoleCtrlHandler function on Windows!
# If posix, save old terminal settings so we can restore them on exit.
try:
    # Only valid for *nix systems.
    import tty, termios
    old_terminal_settings = termios.tcgetattr(sys.stdin)
except:
    # Ok to fail for Windows
    pass

def set_signal_handler(func):
    # If posix, restore old terminal settings.
    if os.name == "nt":
        from ctypes import windll
        windll.kernel32.SetConsoleCtrlHandler(func, True)
    # Install SIGTERM signal handler
    else:
        signal.signal(signal.SIGTERM, func)

# If ctypes present, we have Windows so define the exit with decorators
try:
    import ctypes
    @ctypes.WINFUNCTYPE(ctypes.c_int, ctypes.c_uint)
    def on_exit(signal, func=None):
        logging.info("Failover console stopped with SIGTERM.")
        sys.exit(0)
except:
    def on_exit(signal, func=None):
        if os.name == "posix":
            termios.tcsetattr(sys.stdin, termios.TCSADRAIN,
                              old_terminal_settings)
        logging.info("Failover console stopped with SIGTERM.")
        sys.exit(0)

set_signal_handler(on_exit)

# Setup the command parser
parser = UtilitiesParser(
    version=VERSION_FRM.format(program=os.path.basename(sys.argv[0])),
    description=DESCRIPTION,
    usage=USAGE,
    add_help_option=False)

# Default option to provide help information
parser.add_option("--help", action="help", help="display this help message "
                  "and exit")

# Setup utility-specific options:
add_failover_options(parser)

# Interval for continuous mode
parser.add_option("--interval", "-i", action="store", dest="interval",
                  type="int", default="15", help="interval in seconds for "
                  "polling the master for failure and reporting health. "
                  "Default = 15 seconds. Lowest value is 5 seconds.")

# Add failover modes
parser.add_option("--failover-mode", "-f", action="store", dest="failover_mode",
                  type="choice", default="auto", choices=["auto", "elect",
                  "fail"], help="action to take when the master "
                  "fails. 'auto' = automatically fail to best slave, "
                  "'elect' = fail to candidate list or if no candidate meets "
                  "criteria fail, 'fail' = take no action and stop when master "
                  "fails. Default = 'auto'.")

# Add failover detection extension point
parser.add_option("--exec-fail-check", action="store", dest="exec_fail",
                  type="string", default=None, help="name of script to "
                      "execute on each interval to invoke failover")

# Add force to override registry entry
parser.add_option("--force", action="store_true", dest="force", default=False,
                  help="override the registration check on master for "
                  "multiple instances of the console monitoring the same "
                  "master.")

# Add refresh script external point
parser.add_option("--exec-post-failover", action="store", dest="exec_post_fail",
                  type="string", default=None, help="name of script to "
                  "execute after failover is complete and the utility has "
                  "refreshed the health report.")


# Add rediscover on interval
parser.add_option("--rediscover", action="store_true", dest="rediscover",
                  help="rediscover slaves on interval. Allows console to "
                  "detect when slaves have been removed or added.")

# Pedantic mode for failing if some inconsistencies are found
parser.add_option("-p", "--pedantic", action="store_true", default=False,
                  dest="pedantic", help="fail if some inconsistencies are "
                  "found (e.g. errant transactions on slaves).")

# Add no keyboard input
parser.add_option("--no-keyboard", action="store_true", default=False,
                  dest="no_keyboard", help="start with no keyboard input "
                  "support.")

# Add option to run as daemon
parser.add_option("--daemon", action="store", dest="daemon", default=None,
                  help="run on daemon mode. It can be start, stop, restart "
                  "or nodetach.",
                  type="choice", choices=("start", "stop", "restart",
                                          "nodetach"))

# Add pidfile for the daemon option
parser.add_option("--pidfile", action="store", dest="pidfile",
                  type="string", default=None, help="pidfile for running "
                  "mysqlfailover as a daemon.")

# Add report values for daemon mode
parser.add_option("--report-values", action="store", dest="report_values",
                  type="string", default="health",
                  help="report values used in mysqlfailover running as a "
                  "daemon. It can be health, gtid or uuid. Multiple values "
                  "can be used separated by commas. The default is health.")

# Add verbosity mode
add_verbosity(parser, False)

# Replication user and password
add_rpl_user(parser, None)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Check slaves list
if opt.daemon != "stop":
    check_server_lists(parser, opt.master, opt.slaves)

# Check for errors
if int(opt.interval) < 5:
    parser.error("The --interval option requires a value greater than or "
                 "equal to 5.")

# The value for --timeout needs to be an integer > 0.
try:
    if int(opt.timeout) <= 0:
        parser.error("The --timeout option requires a value greater than 0.")
except ValueError:
    parser.error("The --timeout option requires an integer value.")

# if opt.master is None and opt.daemon and opt.daemon != "stop":
if opt.master is None and opt.daemon != "stop":
    parser.error("You must specify a master to monitor.")

if opt.slaves is None and opt.discover is None and opt.daemon != "stop":
    parser.error("You must supply a list of slaves or the "
                 "--discover-slaves-login option.")

if opt.failover_mode == 'elect' and opt.candidates is None:
    parser.error("Failover mode = 'elect' requires at least one candidate.")

# Parse the master, slaves, and candidates connection parameters
try:
    master_val, slaves_val, candidates_val = parse_failover_connections(opt)
except UtilRplError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Check hostname alias
for slave_val in slaves_val:
    if check_hostname_alias(master_val, slave_val):
        parser.error("The master and one of the slaves are the same host and port.")
for cand_val in candidates_val:
    if check_hostname_alias(master_val, cand_val):
        parser.error("The master and one of the candidates are the same host and port.")

# Create dictionary of options
options = {
    'candidates'       : candidates_val,
    'ping'             : 3 if opt.ping is None else opt.ping,
    'verbosity'        : 0 if opt.verbosity is None else opt.verbosity,
    'before'           : opt.exec_before,
    'after'            : opt.exec_after,
    'fail_check'       : opt.exec_fail,
    'max_position'     : opt.max_position,
    'max_delay'        : opt.max_delay,
    'discover'         : opt.discover,
    'timeout'          : int(opt.timeout),
    'interval'         : opt.interval,
    'failover_mode'    : opt.failover_mode,
    'logging'          : opt.log_file is not None,
    'log_file'         : opt.log_file,
    'force'            : opt.force,
    'post_fail'        : opt.exec_post_fail,
    'rpl_user'         : opt.rpl_user,
    'rediscover'       : opt.rediscover,
    'pedantic'         : opt.pedantic,
    'no_keyboard'      : opt.no_keyboard,
    'daemon'           : opt.daemon,
    'pidfile'          : opt.pidfile,
    'report_values'    : opt.report_values,
    'script_threshold' : opt.script_threshold,
}

# Purge log file of old data
if opt.log_file is not None and not purge_log(opt.log_file, opt.log_age):
    parser.error("Error purging log file.")

# Setup log file
logging.basicConfig(filename=opt.log_file, level=logging.INFO,
                    format='%(asctime)s %(levelname)s %(message)s',
                    datefmt=_DATE_FORMAT)

# Warn user about script threshold checking.
if opt.script_threshold:
    print(SCRIPT_THRESHOLD_WARNING)

# Check if the values specified for the --report-values option are valid.
for report in opt.report_values.split(','):
    if report.lower() not in ("health", "gtid", "uuid"):
        parser.error("The value for the option --report-values is not valid: "
                     "'{0}', the values allowed are 'health', 'gitd' or 'uuid'"
                     "".format(opt.report_values))

# Check the daemon options
if opt.daemon:
    # Check if a POSIX system
    if os.name != "posix":
        parser.error("Running mysqlfailover with --daemon is only available "
                     "for POSIX systems.")

    # Check the presence of --log
    if opt.daemon != "stop" and not opt.log_file:
        parser.error("The option --log is required when using --daemon.")

    # Test pidfile
    if opt.daemon != "nodetach":
        pidfile = opt.pidfile or "./failover_daemon.pid"
        pidfile = os.path.realpath(os.path.normpath(pidfile))
        if opt.daemon == "start":
            # Test if pidfile exists
            if os.path.exists(pidfile):
                parser.error("pidfile {0} already exists. The daemon is "
                             "already running?".format(pidfile))
            # Test if pidfile is writable
            try:
                with open(pidfile, "w") as f:
                    f.write("{0}\n".format(0))
                # Delete temporary pidfile
                os.remove(pidfile)
            except IOError as err:
                parser.error("Unable to write pidfile: {0}"
                             "".format(err.strerror))
        else:
            # opt.daemon == stop/restart, test if pidfile is readable
            pid = None
            try:
                if not os.path.exists(pidfile):
                    parser.error("pidfile {0} does not exist.".format(pidfile))
                with open(pidfile, "r") as f:
                    pid = int(f.read().strip())
            except IOError:
                pid = None
            except ValueError:
                pid = None
            # Test pid presence
            if not pid:
                parser.error("Can not read pid from pidfile.")

if opt.pidfile and not opt.daemon:
    parser.error("The option --daemon is required when using --pidfile.")

try:
    rpl_cmds = RplCommands(master_val, slaves_val, options)
    if opt.daemon:
        rpl_cmds.auto_failover_as_daemon()
    else:
        rpl_cmds.auto_failover(opt.interval)
except UtilError:
    _, e, _ = sys.exc_info()
    # log the error in case it was an usual exception
    logging.log(logging.CRITICAL, e.errmsg.strip(' '))
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)
except KeyboardInterrupt:
    sys.exit(0)

sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the mysql utilities client.
"""

from mysql.utilities.common.options import UtilitiesParser
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import sys

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

try:
    from mysql.utilities.command.utilitiesconsole import UtilitiesConsole
    from mysql.utilities import VERSION_FRM, VERSION_STRING, COPYRIGHT_FULL
    from mysql.utilities.common.options import add_verbosity, check_verbosity
    from mysql.utilities.exception import UtilError
except:
    print("ERROR: MySQL Utilities are either not installed or are not "
          "accessible from this terminal.")
    sys.exit(2)

# Constants
NAME = "MySQL Utilities Client - mysqluc "
DESCRIPTION = "mysqluc - Command line client for running MySQL Utilities"
USAGE = "%prog "
WELCOME_MESSAGE = """
Welcome to the MySQL Utilities Client (mysqluc) version {0}\n{1}
Type 'help' for a list of commands or press TAB twice for list of utilities.
"""
GOODBYE_MESSAGE = "\nThanks for using the MySQL Utilities Client!\n"
PRINT_WIDTH = 75
UTIL_PATH = "/scripts"

def build_variable_dictionary_list(args):
    """Build a variable dictionary from the arguments

    Returns list - list of variables
    """
    variables = []
    arguments = list(args)
    for arg in arguments[:]:
        if '=' in arg:
            try:
                name, value = arg.split('=')
                if not value:
                    raise ValueError
            except ValueError:
                parser.error("Invalid argument assignment: {0}. Please check "
                             "your command.".format(arg))
            variables.append({'name': name, 'value': value})
            arguments.remove(arg)

    if len(arguments) > 0:
        parser.error("Unbalanced arguments. Please check your command.")
    for i in range(0, len(arguments), 2):
        variables.append({'name': arguments[i], 'value': arguments[i+1]})
    return variables

# Setup the command parser
parser = UtilitiesParser(
    version=VERSION_FRM.format(program=os.path.basename(sys.argv[0])),
    description=DESCRIPTION,
    usage=USAGE,
    add_help_option=False)

# Default option to provide help information
parser.add_option("--help", action="help", help="display this help message "
                  "and exit")

# Add display width option
parser.add_option("--width", action="store", dest="width",
                  type="int", help="display width",
                  default=PRINT_WIDTH)

# Add utility directory option
parser.add_option("--utildir", action="store", dest="utildir",
                  type="string", help="location of utilities",
                  default=UTIL_PATH)

# Add execute mode
parser.add_option("-e", "--execute", action="store", dest="commands",
                  type="string", help="execute commands and exit. Multiple "
                  "commands are separated with semi-colons. Note: some "
                  "platforms may require double quotes around command list.",
                  default=None)

# Add utility extra_utilities option
parser.add_option("--add-utility", action="append", dest="add_util",
                  help="append an utility in the format"
                  "mysql<utility_name>. The mysql<utility_name>.py must be "
                  "located inside the folder given by the utildir option",
                  default=[])

# Add utility extra_utilities option
parser.add_option("--hide-utils", action="store_true",
                  dest="hide_util", help="when this option is given,"
                  " the default utilities will not be available, must be used"
                  " only along of --add-utility option", default=False)

# Add verbosity mode
add_verbosity(parser, True)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

if opt.verbosity is None:
    verbosity = 0
else:
    verbosity = opt.verbosity

quiet = opt.quiet
if opt.quiet is None:
    quiet = False

if opt.hide_util and not opt.add_util:
    # TODO: move to common/messages.py
    parser.error("You can only use --hide_utils option along the "
                 "--add-util option.")

extra_utils_dict = {}
for utility in opt.add_util:
    extra_utils_dict[utility] = ()


options = {
    'verbosity' : verbosity,
    'quiet'     : quiet,
    'width'     : opt.width,
    'utildir'   : opt.utildir,
    'variables' : build_variable_dictionary_list(args),
    'prompt'    : 'mysqluc> ',
    'welcome'   : WELCOME_MESSAGE.format(VERSION_STRING, COPYRIGHT_FULL),
    'goodbye'   : GOODBYE_MESSAGE,
    'commands'  : opt.commands,
    'custom'    : True, # We are using custom commands
    'hide_util' : opt.hide_util,
    'add_util'  : extra_utils_dict
}

try:
    print("Launching console ...")
    util_con = UtilitiesConsole(options)
    util_con.run_console()
except KeyboardInterrupt:
    print(options['goodbye'])
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)

sys.exit()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import os
import unittest

from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import get_absolute_path
from mysql.utilities.exception import FormatError

def _spec(info):
    result = []

    user = info['user']
    if ':' in user or '@' in user:
        user = u"'{0}'".format(user)
    result.append(user)

    passwd = info.get('passwd')
    if passwd:
        result.append(':')
        if ':' in passwd or '@' in passwd:
            passwd = u"'{0}'".format(passwd)
        result.append(passwd)

    result.append('@')
    result.append(info['host'])
    if info.get('port'):
        result.append(':')
        result.append(str(info['port']))
    if info.get('unix_socket'):
        result.append(':')
        result.append(info['unix_socket'])
        
    return ''.join(result)


class TestParseConnection(unittest.TestCase):
    """Test that parsing connection strings work correctly.
    """
    # This list consists of input specifier and expected output
    # specifier. If the element is a simple string, the input is
    # expected as output.
    valid_specifiers = [
        ('mats@localhost', 'mats@localhost:3306'),
        ('mats@localhost:3307', 'mats@localhost:3307'), 
        ('mats:foo@localhost', 'mats:foo@localhost:3306'),
        ('mats:foo@localhost:3308', 'mats:foo@localhost:3308'),
        ('mats@localhost:3308:/usr/var/mysqld.sock', 'mats@localhost:3308:/usr/var/mysqld.sock'),
        ('mats:@localhost', 'mats@localhost:3306'),
        ('mysql-user:!#$-%&@localhost', 'mysql-user:!#$-%&@localhost:3306'),
        ('"nuno:mariz":foo@localhost', "'nuno:mariz':foo@localhost:3306"),
        ("nmariz:'foo:bar'@localhost", "nmariz:'foo:bar'@localhost:3306"),
        ("nmariz:'foo@bar'@localhost", "nmariz:'foo@bar'@localhost:3306"),
        ("nmariz:foo'bar@localhost", "nmariz:foo'bar@localhost:3306"),
        ("foo'bar:nmariz@localhost", "foo'bar:nmariz@localhost:3306"),
        ('nmariz:foo"bar@localhost', 'nmariz:foo"bar@localhost:3306'),
        ('foo"bar:nmariz@localhost', 'foo"bar:nmariz@localhost:3306'),
        (u'ɱysql:unicode@localhost', u'ɱysql:unicode@localhost:3306'),
        # IPv6 strings
        ("cbell@3ffe:1900:4545:3:200:f8ff:fe21:67cf", "cbell@[3ffe:1900:4545:3:200:f8ff:fe21:67cf]:3306"),
        ("cbell@fe80:0000:0000:0000:0202:b3ff:fe1e:8329", "cbell@[fe80:0000:0000:0000:0202:b3ff:fe1e:8329]:3306"),
        ("cbell@fe80:0:0:0:202:b3ff:fe1e:8329", "cbell@[fe80:0:0:0:202:b3ff:fe1e:8329]:3306"),
        ("cbell@'fe80::202:b3ff:fe1e:8329'", "cbell@fe80::202:b3ff:fe1e:8329:3306"),
        ("cbell@'fe80::0202:b3ff:fe1e:8329'", "cbell@fe80::0202:b3ff:fe1e:8329:3306"),
        ('cbell@"FE80::0202:B3FF:FE1E:8329"', "cbell@FE80::0202:B3FF:FE1E:8329:3306"),
        ('cbell@1:2:3:4:5:6:7:8', 'cbell@[1:2:3:4:5:6:7:8]:3306'),
        ("cbell@'E3D7::51F4:9BC8:192.168.100.32'", 'cbell@E3D7::51F4:9BC8:192.168.100.32:3306'),
        ("cbell@'E3D7::51F4:9BC8'", 'cbell@E3D7::51F4:9BC8:3306'),
        ]

    # These specifiers are invalid and should generate a FormatError.
    invalid_specificers = [
        'mats@', '@localhost', 'cbell@what:is::::this?',
        'cbell@1:2:3:4:5:6:192.168.1.110',
        'cbell@E3D7::51F4:9BC8:192.168.100.32',
        ]
    
    def test_valid(self):
        """Test parsing valid versions of connection strings.
        """
        for source, expected in self.valid_specifiers:
            result = _spec(parse_connection(source))
            frm = u"{0}: was {1}, expected {2}"
            msg = frm.format(source, result, expected)
            self.assertEqual(expected, result, msg)

    def test_invalid(self):
        """Test parsing invalid versions of connection strings.

        If the connection string is invalid, a FormatError should be thrown.
        """
        for spec in self.invalid_specificers:
            try:
                parse_connection(spec)
            except FormatError:
                pass
            except:
                self.fail("Unexpected exception thrown.")
            else:
                self.fail("Exception not thrown for: '%s'." % spec)


@unittest.skipUnless(os.name == 'posix', 'Requires POSIX')
class TestPosixPath(unittest.TestCase):
    """Test posix paths.
    """

    paths = [
        ('/home/user/a/', '/home/user/a'),
        ('/home/user/b/../a', '/home/user/a')
    ]

    def setUp(self):
        env_home = os.environ.get('HOME', '')
        abs_path = os.path.abspath('.')
        self.paths.extend([
            ('a//b/../c', '{0}/a/c'.format(abs_path)),
            ('a/./b', '{0}/a/b'.format(abs_path)),
            ('{0}/a/d/../b/c'.format(env_home), '{0}/a/b/c'.format(env_home)),
            ('~/a/b', '{0}/a/b'.format(env_home)),
            ('~/a/../b', '{0}/b'.format(env_home)),
        ])

    def test_paths(self):
        """Test POSIX paths.
        """
        for source, expected in self.paths:
            self.assertEqual(get_absolute_path(source), expected)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the object diff utility which allows users to compare the
definitions of two objects and return the difference (like diff).
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import re
import sys

from mysql.utilities.command.diff import object_diff, database_diff
from mysql.utilities.common.messages import PARSE_ERR_DB_OBJ_MISSING
from mysql.utilities.common.messages import PARSE_ERR_DB_OBJ_MISSING_MSG
from mysql.utilities.common.messages import PARSE_ERR_DB_OBJ_PAIR
from mysql.utilities.common.messages import PARSE_ERR_DB_OBJ_PAIR_EXT
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_difftype
from mysql.utilities.common.options import add_verbosity, check_verbosity
from mysql.utilities.common.options import add_changes_for, add_reverse
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqldiff "
DESCRIPTION = "mysqldiff - compare object definitions among objects" + \
              " where the difference is how db1.obj1 differs from db2.obj2"
USAGE = "%prog --server1=user:pass@host:port:socket " + \
        "--server2=user:pass@host:port:socket db1.object1:db2.object1 db3:db4"
PRINT_WIDTH = 75

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, False, False)

# Connection information for the source server
parser.add_option("--server1", action="store", dest="server1",
                  type="string", default="root@localhost:3306",
                  help="connection information for first server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Connection information for the destination server
parser.add_option("--server2", action="store", dest="server2",
                  type="string", default=None,
                  help="connection information for second server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Add display width option
parser.add_option("--width", action="store", dest="width",
                  type="int", help="display width",
                  default=PRINT_WIDTH)

# Force mode
parser.add_option("--force", action="store_true", dest="force",
                  help="do not abort when a diff test fails")

# Skip check of table options.
parser.add_option("--skip-table-options", action="store_true",
                  dest="skip_tbl_opts",
                  help="skip check of all table options (e.g., "
                       "AUTO_INCREMENT, ENGINE, CHARSET, etc.).")

# Add verbosity and quiet (silent) mode
add_verbosity(parser, True)

# Add difftype option with SQL option
add_difftype(parser, True)

# Add the direction (changes-for)
add_changes_for(parser, None)

# Add show reverse option
add_reverse(parser)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

# Set options for database operations.
options = {
    "quiet"            : opt.quiet,
    "verbosity"        : opt.verbosity,
    "difftype"         : opt.difftype,
    "force"            : opt.force,
    "width"            : opt.width,
    "changes-for"      : opt.changes_for,
    "reverse"          : opt.reverse,
    "skip_table_opts"  : opt.skip_tbl_opts,
}

# Parse server connection values
try:
    server1_values = parse_connection(opt.server1, None, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Server1 connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Server1 connection values invalid: %s." % err.errmsg)
if opt.server2 is not None:
    try:
        server2_values = parse_connection(opt.server2, None, options)
    except FormatError:
        _, err, _ = sys.exc_info()
        parser.error("Server2 connection values invalid: %s." % err)
    except UtilError:
        _, err, _ = sys.exc_info()
        parser.error("Server2 connection values invalid: %s." % err.errmsg)
else:
    server2_values = None

# Check for arguments
if len(args) == 0:
    parser.error("No objects specified to compare.")

# run the diff
diff_failed = False
arg_regexp = re.compile('{0}(?:\:){0}'.format(REGEXP_QUALIFIED_OBJ_NAME))
for argument in args:
    m_obj = arg_regexp.match(argument)
    if not m_obj:
        parser.error(PARSE_ERR_DB_OBJ_PAIR.format(db_obj_pair=argument,
                                                  db1_label='db1',
                                                  obj1_label='object1',
                                                  db2_label='db2',
                                                  obj2_label='object2'))
    db1, obj1, db2, obj2 = m_obj.groups()

    # Verify if the size of the objects matched by the REGEX is equal to the
    # initial specified string. In general, this identifies the missing use
    # of backticks.
    matched_size = len(db1)
    if obj1:
        # add 1 for the separator '.'
        matched_size = matched_size + 1
        matched_size = matched_size + len(obj1)
    # add 1 for the separator ':'
    matched_size = matched_size + 1
    matched_size = matched_size + len(db2)
    if obj2:
        # add 1 for the separator '.'
        matched_size = matched_size + 1
        matched_size = matched_size + len(obj2)
    if matched_size != len(argument):
        parser.error(PARSE_ERR_DB_OBJ_PAIR_EXT.format(db_obj_pair=argument,
                                                      db1_label='db1',
                                                      obj1_label='object1',
                                                      db2_label='db2',
                                                      obj2_label='object2',
                                                      db1_value=db1,
                                                      obj1_value=obj1,
                                                      db2_value=db2,
                                                      obj2_value=obj2))

    if (obj1 and not obj2) or (not obj1 and obj2):
        if obj1:
            detail = PARSE_ERR_DB_OBJ_MISSING.format(db_no_obj_label='db2',
                                                     db_no_obj_value=db2,
                                                     only_obj_value=obj1,
                                                     db_obj_label='db1',
                                                     db_obj_value=db1)
        else:
            detail = PARSE_ERR_DB_OBJ_MISSING.format(db_no_obj_label='db1',
                                                     db_no_obj_value=db1,
                                                     only_obj_value=obj2,
                                                     db_obj_label='db2',
                                                     db_obj_value=db2)
        parser.error(PARSE_ERR_DB_OBJ_MISSING_MSG.format(detail=detail,
                                                         db1_label='db1',
                                                         obj1_label='object1',
                                                         db2_label='db2',
                                                         obj2_label='object2'))

    # We have db1.obj:db2.obj
    if obj1:
        try:
            diff = object_diff(server1_values, server2_values,
                               "%s.%s" % (db1, obj1),
                               "%s.%s" % (db2, obj2), options)
        except UtilError:
            _, e, _ = sys.exc_info()
            print("ERROR: %s" % e.errmsg)
            sys.exit(1)
        if diff is not None:
            diff_failed = True

    # We have db1:db2
    else:
        try:
            res = database_diff(server1_values, server2_values,
                                db1, db2, options)
        except UtilError:
            _, e, _ = sys.exc_info()
            print("ERROR: %s" % e.errmsg)
            sys.exit(1)
        if not res:
            diff_failed = True

if diff_failed:
    if not opt.quiet:
        print("Compare failed. One or more differences found.")
    sys.exit(1)

if not opt.quiet:
    print("Success. All objects are the same.")

sys.exit()



#
# Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import unittest

from mysql.utilities.common.tools import requires_encoding, encode
from mysql.utilities.common.tools import requires_decoding, decode

class TestDecodeEncode(unittest.TestCase):
    """Test that decode and encode strings work correctly.
    """
    # This list consists of decoded string and expected encoded string
    test_cases = [
        ('this.has.periods', 'this@002ehas@002eperiods'),
        ('me.too.periods', 'me@002etoo@002eperiods'),
        ('abc.kk-d.e-f', 'abc@002ekk@002dd@002ee@002df'),
        # sanity check to show this does not need encoding/decoding
        ('sanity$che_ck', 'sanity$che_ck'),
    ]

    def test_decode_encode(self):
        """Test valid encode/decode conversions of strings.
        """
        for enc_str, dec_str in self.test_cases:
            frm = "{0}: was {1}, expected {2}"
            # First, do encode
            result = encode(enc_str)
            msg = frm.format(enc_str, result, dec_str)
            self.assertEqual(dec_str, result, msg)
            # Now, do decode
            result = decode(dec_str)
            msg = frm.format(enc_str, result, dec_str)
            self.assertEqual(enc_str, result, msg)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the show replication topology utility. It is used to
find the slaves for a given master and can traverse the list of slaves
checking for additional master/slave connections.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.exception import UtilError
from mysql.utilities.common.options import add_verbosity, setup_common_options
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.command.show_rpl import show_topology
from mysql.utilities.exception import FormatError

# Constants
NAME = "MySQL Utilities - mysqlrplshow "
DESCRIPTION = "mysqlrplshow - show slaves attached to a master"
USAGE = "%prog --master=root@localhost:3306 "

PRINT_WIDTH = 75

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, True, False)

# Setup utility-specific options:

# Connection information for the source server
parser.add_option("--master", action="store", dest="master",
                  type="string", default="root@localhost:3306",
                  help="connection information for master server in " + \
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Show graph option
parser.add_option("-l", "--show-list", action="store_true", dest="show_list",
                  help="print a list of the topology.", default=False)

# Output format
add_format_option(parser, "display the list in either grid (default), "
                  "tab, csv, or vertical format", "grid")

# Check slaves option - if True, recurse slaves from master to find
# additional master/slave connections
parser.add_option("-r", "--recurse", action="store_true",
                  dest="recurse",
                  help="traverse the list of slaves to find additional "
                  "master/slave connections. User this option to map a "
                  "replication topology.", default=False)

# Add limit for recursion
parser.add_option("--max-depth", action="store", default=None, type="int",
                  help="limit the traversal to this depth. Valid only with "
                  "the --recurse option. Valid values are non-negative "
                  "integers.", dest="max_depth")

# Prompt for slave connections if default login/password fail
parser.add_option("-p", "--prompt", action="store_true", dest="prompt",
                  help="prompt for slave user and password if different from "
                  "master login.", default=False)

# Number of retries for failed slave login
parser.add_option("-n", "--num-retries", action="store", dest="num_retries",
                  type="int", help="number of retries allowed for failed "
                  "slave login attempt. Valid with --prompt only.",
                  default=0)

# Add verbosity mode and quiet option
add_verbosity(parser, True)

parser.add_option("--discover-slaves-login", action="store", dest="discover",
                  default=None, type="string", help="at startup, query "
                  "master for all registered slaves and use the user name "
                  "and password specified to connect. Supply the user and "
                  "password in the form <user>[:<password>] or <login-path>. "
                  "For example, --discover-slaves-login=joe:secret will use "
                  "'joe' as the user and 'secret' as the password for each "
                  "discovered slave.")

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Fail is --discover-slaves-login not specified
if opt.discover is None:
    parser.error("The --discover-slaves-login is required to test slave "
                 "connectivity.")

# Fail if recurse specified and max-depth is invalid
if opt.recurse and opt.max_depth is not None:
    if opt.max_depth < 0:
        parser.error("The --max-depth option needs to be >= 0.")

# Parse master connection values
try:
    m_values = parse_connection(opt.master, None, opt)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err.errmsg)

# Create dictionary of options
options = {
    'quiet'          : opt.quiet,
    'prompt'         : opt.prompt,
    'num_retries'    : opt.num_retries,
    'recurse'        : opt.recurse,
    'show_list'      : opt.show_list,
    'format'         : opt.format,
    'max_depth'      : opt.max_depth,
    'discover'       : opt.discover,
    'verbosity'      : opt.verbosity
}

try:
    res = show_topology(m_values, options)
    if res:
        sys.exit(1)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the replication slave administration utility. It is used to
perform replication operations on one or more slaves.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import logging
import os.path
import sys

from mysql.utilities.exception import UtilError, UtilRplError
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.messages import SCRIPT_THRESHOLD_WARNING
from mysql.utilities.common.options import add_format_option, add_verbosity
from mysql.utilities.common.options import add_failover_options, add_rpl_user
from mysql.utilities.common.options import check_server_lists
from mysql.utilities.common.options import CaseInsensitiveChoicesOption
from mysql.utilities.common.messages import (PARSE_ERR_OPT_INVALID_CMD_TIP,
                                             PARSE_ERR_OPTS_REQ_BY_CMD,
                                             PARSE_ERR_SLAVE_DISCO_REQ,
                                             PARSE_ERR_SLAVE_DISCO_EXC,
                                             WARN_OPT_NOT_REQUIRED,
                                             WARN_OPT_NOT_REQUIRED_ONLY_FOR,
                                             ERROR_SAME_MASTER,
                                             ERROR_MASTER_IN_SLAVES,
                                             SLAVES, CANDIDATES)
from mysql.utilities.common.options import UtilitiesParser
from mysql.utilities.common.server import Server, check_hostname_alias
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.common.topology import parse_failover_connections
from mysql.utilities.command.rpl_admin import RplCommands, purge_log
from mysql.utilities.command.rpl_admin import get_valid_rpl_commands
from mysql.utilities.command.rpl_admin import get_valid_rpl_command_text
from mysql.utilities.exception import FormatError
from mysql.utilities import VERSION_FRM


class MyParser(UtilitiesParser):
    def format_epilog(self, formatter):
        return self.epilog

# Constants
NAME = "MySQL Utilities - mysqlrpladmin "
DESCRIPTION = "mysqlrpladmin - administration utility for MySQL replication"
USAGE = "%prog --slaves=root@localhost:3306 <command>"
_DATE_FORMAT = '%Y-%m-%d %H:%M:%S %p'

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = MyParser(
    version=VERSION_FRM.format(program=os.path.basename(sys.argv[0])),
    description=DESCRIPTION,
    usage=USAGE,
    add_help_option=False,
    option_class=CaseInsensitiveChoicesOption,
    epilog=get_valid_rpl_command_text())

# Default option to provide help information
parser.add_option("--help", action="help", help="display this help message "
                  "and exit")

# Setup utility-specific options:
add_failover_options(parser)

# Connection information for the master
# Connect information for candidate server for switchover
parser.add_option("--new-master", action="store", dest="new_master", default=None,
                  type="string", help="connection information for the "
                  "slave to be used to replace the master for switchover, "
                  "in the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>]. Valid only with "
                  "switchover command.")

# Force the execution of the command, ignoring some errors
parser.add_option("--force", action="store_true", dest="force",
                  help="ignore prerequisite check results or some "
                  "inconsistencies found (e.g. errant transactions on slaves) "
                  "and execute action")

# Output format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", None)

# Add demote option
parser.add_option("--demote-master", action="store_true", dest="demote",
                  help="make master a slave after switchover.")

# Add no-health option
parser.add_option("--no-health", action="store_true", dest="no_health",
                  help="turn off health report after switchover or failover.")

# Add verbosity mode
add_verbosity(parser, True)

# Replication user and password
add_rpl_user(parser, None)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Check for invalid command
if len(args) > 1:
    parser.error("You can only specify one command to execute at a time.")
elif len(args) == 0:
    parser.error("You must specify a command to execute.")

# At least one of the options --discover-slaves-login or --slaves is required.
if not opt.discover and not opt.slaves:
    parser.error(PARSE_ERR_SLAVE_DISCO_REQ)

# --discover-slaves-login and --slaves cannot be used simultaneously (only one)
if opt.discover and opt.slaves:
    parser.error(PARSE_ERR_SLAVE_DISCO_EXC)

# Check slaves list
check_server_lists(parser, opt.master, opt.slaves)

# The value for --timeout needs to be an integer > 0.
try:
    if int(opt.timeout) <= 0:
        parser.error("The --timeout option requires a value greater than 0.")
except ValueError:
    parser.error("The --timeout option requires an integer value.")

# Check errors and warnings of options and combinations.

command = args[0].lower()
if not command in get_valid_rpl_commands():
    parser.error("'%s' is not a valid command." % command)

# --master and --new-master options are required by 'switchover'
if command == 'switchover' and (not opt.new_master or not opt.master):
    req_opts = '--master and --new-master'
    parser.error(PARSE_ERR_OPTS_REQ_BY_CMD.format(cmd=command, opts=req_opts))

# --master and either --slaves or --discover-slaves-login options are required
# by 'elect', 'health' and 'gtid'
if command in ['elect', 'health', 'gtid'] and not opt.master and \
   (not opt.slaves or not opt.discover):
    req_opts = '--master and either --slaves or --discover-slaves-login'
    parser.error(PARSE_ERR_OPTS_REQ_BY_CMD.format(cmd=command, opts=req_opts))

# --slaves options are required by 'start', 'stop' and 'reset'
# --master is optional
if command in ['start', 'stop', 'reset'] and not opt.slaves:
    req_opts = '--slaves'
    parser.error(PARSE_ERR_OPTS_REQ_BY_CMD.format(cmd=command, opts=req_opts))

# Validate the required options for the failover command
if command == 'failover':
    # --discover-slaves-login is invalid (as it will require a master)
    # instead --slaves needs to be used.
    if opt.discover:
        invalid_opt = '--discover-slaves-login'
        parser.error(PARSE_ERR_OPT_INVALID_CMD_TIP.format(opt=invalid_opt,
                                                          cmd=command,
                                                          opt_tip='--slaves'))
    # --master will be ignored
    if opt.master:
        print(WARN_OPT_NOT_REQUIRED.format(opt='--master', cmd=command))
        opt.master = None

# --ping only used by 'health' command
if opt.ping and not command == 'health':
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--ping', cmd=command,
                                                only_cmd='health'))
    opt.ping = None

# --exec-after only used by 'failover' or 'switchover' command
if opt.exec_after and command not in ['switchover', 'failover']:
    only_used_cmds = 'failover or switchover'
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--exec-after',
                                                cmd=command,
                                                only_cmd=only_used_cmds))
    opt.exec_after = None

# --exec-before only used by 'failover' or 'switchover' command
if opt.exec_before and command not in ['switchover', 'failover']:
    only_used_cmds = 'failover or switchover'
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--exec-before',
                                                cmd=command,
                                                only_cmd=only_used_cmds))
    opt.exec_before = None

# --new-master only required for 'switchover' command
if opt.new_master and command != 'switchover':
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--new-master',
                                                cmd=command,
                                                only_cmd='switchover'))
    opt.new_master = None

# --candidates only used by 'failover' or 'elect' command
if opt.candidates and command not in ['elect', 'failover']:
    only_used_cmds = 'failover or elect'
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--candidates',
                                                cmd=command,
                                                only_cmd=only_used_cmds))
    opt.candidates = None

# --format only used by 'health' or 'gtid' command
if opt.format and command not in ['health', 'gtid']:
    only_used_cmds = 'health or gtid'
    print(WARN_OPT_NOT_REQUIRED_ONLY_FOR.format(opt='--format',
                                                cmd=command,
                                                only_cmd=only_used_cmds))
    opt.format = None

# Parse the --new-master connection string
if opt.new_master:
    try:
        new_master_val = parse_connection(opt.new_master, None, opt)
    except FormatError:
        _, err, _ = sys.exc_info()
        parser.error("New master connection values invalid: %s." % err)
    except UtilError:
        _, err, _ = sys.exc_info()
        parser.error("New master connection values invalid: %s." % err.errmsg)
else:
    new_master_val = None

# Parse the master, slaves, and candidates connection parameters
try:
    master_val, slaves_val, candidates_val = parse_failover_connections(opt)
except UtilRplError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Check hostname alias
if new_master_val:
    if check_hostname_alias(master_val, new_master_val):
        master = Server({'conn_info' : master_val})
        new_master = Server({'conn_info' : new_master_val})
        parser.error(ERROR_SAME_MASTER.format(n_master_host=new_master.host,
                                              n_master_port=new_master.port,
                                              master_host=master.host,
                                              master_port=master.port))

if master_val:
    for slave_val in slaves_val:
        if check_hostname_alias(master_val, slave_val):
            master = Server({'conn_info' : master_val})
            slave = Server({'conn_info' : slave_val})
            msg = ERROR_MASTER_IN_SLAVES.format(master_host=master.host,
                                                master_port=master.port,
                                                slaves_candidates=SLAVES,
                                                slave_host=slave.host,
                                                slave_port=slave.port)
            parser.error(msg)
    for cand_val in candidates_val:
        if check_hostname_alias(master_val, cand_val):
            master = Server({'conn_info' : master_val})
            candidate = Server({'conn_info' : cand_val})
            msg = ERROR_MASTER_IN_SLAVES.format(master_host=master.host,
                                                master_port=master.port,
                                                slaves_candidates=CANDIDATES,
                                                slave_host=candidate.host,
                                                slave_port=candidate.port)
            parser.error(msg)

# Create dictionary of options
options = {
    'new_master'       : new_master_val,
    'candidates'       : candidates_val,
    'ping'             : 3 if opt.ping is None else opt.ping,
    'format'           : opt.format,
    'verbosity'        : 0 if opt.verbosity is None else opt.verbosity,
    'before'           : opt.exec_before,
    'after'            : opt.exec_after,
    'force'            : opt.force,
    'max_position'     : opt.max_position,
    'max_delay'        : opt.max_delay,
    'discover'         : opt.discover,
    'timeout'          : int(opt.timeout),
    'demote'           : opt.demote,
    'quiet'            : opt.quiet,
    'logging'          : opt.log_file is not None,
    'log_file'         : opt.log_file,
    'no_health'        : opt.no_health,
    'rpl_user'         : opt.rpl_user,
    'script_threshold' : opt.script_threshold,
}

# If command = HEALTH, turn on --force
if command == 'health' or command == 'gtid':
    options['force'] = True

# Purge log file of old data
if opt.log_file is not None and not purge_log(opt.log_file, opt.log_age):
    parser.error("Error purging log file.")

# Warn user about script threshold checking.
if opt.script_threshold:
    print(SCRIPT_THRESHOLD_WARNING)

# Setup log file
try:
    logging.basicConfig(filename=opt.log_file, level=logging.INFO,
                        format='%(asctime)s %(levelname)s %(message)s',
                        datefmt=_DATE_FORMAT)
except IOError:
    _, e, _ = sys.exc_info()
    parser.error("Error opening log file: %s" % str(e.args[1]))

try:
    rpl_cmds = RplCommands(master_val, slaves_val, options)
    rpl_cmds.execute_command(command, options)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the import database utility which allows users to import
metadata for objects in a database and data for tables.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import sys
import time
import re

from mysql.utilities.command import dbimport
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import setup_common_options, add_engines
from mysql.utilities.common.options import add_skip_options, check_skip_options
from mysql.utilities.common.options import add_verbosity, check_verbosity
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqldbimport "
DESCRIPTION = "mysqldbimport - import metadata and data from files"
USAGE = "%prog --server=user:pass@host:port:socket db1.csv db2.sql db3.grid"

_PERMITTED_IMPORTS = ["data", "definitions", "both"]

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

def print_elapsed_time(start_test):
    """Print the elapsed time to stdout (screen)

    start_test[in]      The starting time of the test
    """
    stop_test = time.time()
    display_time = int((stop_test - start_test) * 100)
    if display_time == 0:
        display_time = 1
    print("Time: %6d\n" % display_time)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE)

# Setup utility-specific options:

# Input format
add_format_option(parser, "the input file format in either sql (default), "
                  "grid, tab, csv, raw_csv or vertical format", "sql", True,
                  extra_formats=["raw_csv"])

# Import mode
parser.add_option("-i", "--import", action="store", dest="import_type",
                  default="definitions", help="control the import of either "
                  "'data' = only the table data for the tables in the database "
                  "list, 'definitions' = import only the definitions for "
                  "the objects in the database list, or 'both' = import "
                  "the metadata followed by the data "
                  "(default: import definitions)", type="choice",
                  choices=_PERMITTED_IMPORTS)

# Drop mode
parser.add_option("-d", "--drop-first", action="store_true", default=False,
                  help="drop database before importing.", dest="do_drop")

# Single insert mode
parser.add_option("-b", "--bulk-insert", action="store_true",
                  dest="bulk_insert", default=False, help="use bulk insert "
                  "statements for data (default:False)")

# Header row
parser.add_option("-h", "--no-headers", action="store_true", dest="no_headers",
                  default=False, help="files do not contain column headers")

# Dryrun mode
parser.add_option("--dryrun", action="store_true", dest="dryrun",
                  default=False, help="import the files and generate the "
                  "statements but do not execute them - useful for testing "
                  "file validity")

# Add table for import raw csv files
parser.add_option("--table", action="store", dest="table", default=None,
                  help="destination table in the form: <db>.<table>.")

# Skip blobs for import
parser.add_option("--skip-blobs", action="store_true", dest="skip_blobs",
                  default=False, help="do not import blob data.")

# Skip replication commands
parser.add_option("--skip-rpl", action="store_true", dest="skip_rpl",
                  default=False, help="do not execute replication commands.")

# Add skip generation of GTID statements
parser.add_option("--skip-gtid", action="store_true", default=False,
                  dest="skip_gtid", help="do not execute the GTID_PURGED "
                  "statements.")

# Add the skip common options
add_skip_options(parser)

# Add verbosity and quiet (silent) mode
add_verbosity(parser, True)

# Add engine options
add_engines(parser)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

try:
    skips = check_skip_options(opt.skip_objects)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Fail if no arguments
if len(args) == 0:
    parser.error("You must specify at least one file to import.")

if opt.skip_blobs and not opt.import_type == "data":
    print("# WARNING: --skip-blobs option ignored for metadata import.")

if "data" in skips and opt.import_type == "data":
    print("ERROR: You cannot use --import=data and --skip-data when "
          "importing table data.")
    sys.exit(1)

if "create_db" in skips and opt.do_drop:
    print("ERROR: You cannot combine --drop-first and --skip=create_db.")
    exit (1)

# Set options for database operations.
options = {
    "skip_tables"   : "tables" in skips,
    "skip_views"    : "views" in skips,
    "skip_triggers" : "triggers" in skips,
    "skip_procs"    : "procedures" in skips,
    "skip_funcs"    : "functions" in skips,
    "skip_events"   : "events" in skips,
    "skip_grants"   : "grants" in skips,
    "skip_create"   : "create_db" in skips,
    "skip_data"     : "data" in skips,
    "skip_blobs"    : opt.skip_blobs,
    "format"        : opt.format,
    "no_headers"    : opt.no_headers,
    "single"        : not opt.bulk_insert,
    "import_type"   : opt.import_type,
    "dryrun"        : opt.dryrun,
    "do_drop"       : opt.do_drop,
    "quiet"         : opt.quiet,
    "verbosity"     : opt.verbosity,
    "debug"         : opt.verbosity >= 3,
    "new_engine"    : opt.new_engine,
    "def_engine"    : opt.def_engine,
    "skip_rpl"      : opt.skip_rpl,
    "skip_gtid"     : opt.skip_gtid,
    "table"         : opt.table,
}

# Parse server connection values
try:
    server_values = parse_connection(opt.server, None, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err.errmsg)

# Check values for --format=raw_csv
if opt.format == "raw_csv":
    if not opt.table:
        print("ERROR: You must provide --table while using --format=raw_csv.")
        sys.exit(1)
    # Validate table name using format <db>.<table>
    table_re = re.compile(r"{0}(?:\.){0}".format(REGEXP_QUALIFIED_OBJ_NAME))
    if not table_re.match(opt.table):
        parser.error("Invalid table name: {0}.".format(opt.table))

# Ignore --table for formats other than RAW_CSV.
if opt.table and opt.format != "raw_csv":
    print("WARNING: The --table option is only required for --format=raw_csv "
          "(option ignored).")

# Build list of files to import
file_list = []
for file_name in args:
    file_list.append(file_name)

try:
    # record start time
    if opt.verbosity >= 3:
        start_test = time.time()

    for file_name in file_list:
        dbimport.import_file(server_values, file_name, options)

    if opt.verbosity >= 3:
        print_elapsed_time(start_test)

except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the check index utility. It is used to check for
duplicate or redundant indexes for a list of database (operates on
all tables in each database), a list of tables in the for db.table,
or all tables in all databases except internal databases.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.command import indexcheck
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
DESCRIPTION = "mysqlindexcheck - check for duplicate or redundant indexes"
USAGE = "%prog --server=user:pass@host:port:socket db1.table1 db2 db3.table2"

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE)

# Display DROP statements
parser.add_option("--show-drops", "-d", action="store_true",
                  dest="show_drops", default=False,
                  help="display DROP statements for dropping indexes")

# Display all indexes per table
parser.add_option("--show-indexes", "-i", action="store_true",
                  dest="show_indexes", default=False,
                  help="display indexes for each table")

# Force mode
parser.add_option("-s", "--skip", action="store_true", dest="skip",
                  help="skip tables that do not exist",
                  default=False)

# Index list format
add_format_option(parser, "display the list of indexes per table in either "
                  "sql, grid (default), tab, csv, or vertical format", "grid",
                  True)

# Show index statistics
parser.add_option("--stats", action="store_true",
                  dest="stats", default=False,
                  help="show index performance statistics")

# Set limit for best
parser.add_option("--best", action="store",
                  dest="best", default=None,
                  help="limit index statistics to the best N indexes")

# Set limit for worst
parser.add_option("--worst", action="store",
                  dest="worst", default=None,
                  help="limit index statistics to the worst N indexes")

# Add verbosity mode
add_verbosity(parser, False)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Check to make sure at least one table specified.
if len(args) == 0:
    parser.error("You must specify at least one table or database to check.")

# Parse source connection values
try:
    source_values = parse_connection(opt.server)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err.errmsg)

# Check best, worst for validity
best = None
if opt.best is not None:
    try:
        best = int(opt.best)
    except:
        best = -1
if best is not None and best < 1:
    parser.error("The --best parameter must be an integer >= 1")

worst = None
if opt.worst is not None:
    try:
        worst = int(opt.worst)
    except:
        worst = -1
if worst is not None and worst < 1:
    parser.error("The --worst parameter must be an integer >= 1")

if opt.stats and worst is not None and best is not None:
    parser.error("You must specify either --best or --worst but not both.")

# default to worst performing queries
if opt.stats and worst is None and best is None:
    worst = 5

# no stats specified
if (worst is not None or best is not None) and not opt.stats:
    parser.error("You must specify --stats for --best or --worst to take "
                 "effect.")

# Build dictionary of options
options = {
    "show-drops"    : opt.show_drops,
    "skip"          : opt.skip,
    "verbosity"     : opt.verbosity,
    "show-indexes"  : opt.show_indexes,
    "index-format"  : opt.format,
    "stats"         : opt.stats,
    "best"          : best,
    "worst"         : worst
}

try:
    res = indexcheck.check_index(source_values, args, options)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#
# Copyright (c) 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
import sys
import unittest

from mysql.utilities.common.tools import check_connector_python

def connector_python_found():
    try:
        import mysql.connector
    except Exception:
        return True
    return False


@unittest.skipUnless(connector_python_found(),
                     "This test requires C/Py to be inaccessible. "
                     "Please run the test again without C/Py.")
class TestConnectorPython(unittest.TestCase):
    """Test that detection of Connector/Python is working.
    """
    def test_check_connector_python(self):
        """Test valid detection of missing connector/python.
        """
        res = check_connector_python(False)
        self.assertEqual(False, res, "Test found connector/python when it "
                         "should not have!")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the copy database utility which ensures a database
is exactly the same among two servers.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import re
import sys
import time

from mysql.utilities.command import dbcopy
from mysql.utilities.common.messages import PARSE_ERR_DB_PAIR
from mysql.utilities.common.messages import PARSE_ERR_DB_PAIR_EXT
from mysql.utilities.common.my_print_defaults import MyDefaultsReader
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_skip_options
from mysql.utilities.common.options import add_verbosity, check_verbosity
from mysql.utilities.common.options import check_skip_options, add_engines
from mysql.utilities.common.options import add_all, check_all, add_locking
from mysql.utilities.common.options import add_regexp, add_rpl_mode
from mysql.utilities.common.options import check_rpl_options, add_rpl_user
from mysql.utilities.common.sql_transform import is_quoted_with_backticks
from mysql.utilities.common.sql_transform import remove_backtick_quoting
from mysql.utilities.common.tools import check_connector_python

from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqldbcopy "
DESCRIPTION = "mysqldbcopy - copy databases from one server to another"
USAGE = "%prog --source=user:pass@host:port:socket " \
        "--destination=user:pass@host:port:socket orig_db:new_db"

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

def print_elapsed_time(start_test):
    """Print the elapsed time to stdout (screen)

    start_test[in]      The starting time of the test
    """
    stop_test = time.time()
    display_time = int((stop_test - start_test) * 100)
    if display_time == 0:
        display_time = 1
    print("Time: %6d\n" % display_time)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, True, False)

# Setup utility-specific options:

# Connection information for the source server
parser.add_option("--source", action="store", dest="source",
                  type = "string", default="root@localhost:3306",
                  help="connection information for source server in " + \
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Connection information for the destination server
parser.add_option("--destination", action="store", dest="destination",
                  type = "string",
                  help="connection information for destination server in " + \
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Overwrite mode
parser.add_option("-f", "--force", action="store_true", dest="force",
                  help="drop the new database or object if it exists",
                  default=False)

# Threaded/connection mode
parser.add_option("--threads", action="store", dest="threads",
                  default=1, help="use multiple threads (connections) "
                  "for insert")

# Add the exclude database option
parser.add_option("-x", "--exclude", action="append", dest="exclude",
                  type="string", default=None, help="exclude one or more "
                  "objects from the operation using either a specific name "
                  "(e.g. db1.t1), a LIKE pattern (e.g. db1.t% or db%.%) or a "
                  "REGEXP search pattern. To use a REGEXP search pattern for "
                  "all exclusions, you must also specify the --regexp option. "
                  "Repeat the --exclude option for multiple exclusions.")

# Add the all database options
add_all(parser, "databases")

# Add the skip common options
add_skip_options(parser)

# Add verbosity and quiet (silent) mode
add_verbosity(parser, True)

# Add engine options
add_engines(parser)

# Add locking options
add_locking(parser)

# Add regexp
add_regexp(parser)

# Replication user and password
add_rpl_user(parser, None)

# Add replication options but don't include 'both'
add_rpl_mode(parser, False, False)

# Add option to skip GTID generation
parser.add_option("--skip-gtid", action="store_true", default=False,
                  dest="skip_gtid", help="skip creation and execution of "
                  "GTID statements during copy.")

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

try:
    skips = check_skip_options(opt.skip_objects)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Fail if no options listed.
if opt.destination is None:
    parser.error("No destination server specified.")

# Fail if no db arguments or all
if len(args) == 0 and not opt.all:
    parser.error("You must specify at least one database to copy or "
                 "use the --all option to copy all databases.")

# Fail if we have arguments and all databases option listed.
check_all(parser, opt, args, "databases")

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

# Process --exclude values to remove unnecessary quotes (when used) in order
# to avoid further matching issues.
if opt.exclude:
    # Remove unnecessary outer quotes.
    exclude_list = [pattern.strip("'\"") for pattern in opt.exclude]
else:
    exclude_list = opt.exclude

# Set options for database operations.
options = {
    "skip_tables"      : "tables" in skips,
    "skip_views"       : "views" in skips,
    "skip_triggers"    : "triggers" in skips,
    "skip_procs"       : "procedures" in skips,
    "skip_funcs"       : "functions" in skips,
    "skip_events"      : "events" in skips,
    "skip_grants"      : "grants" in skips,
    "skip_create"      : "create_db" in skips,
    "skip_data"        : "data" in skips,
    "force"            : opt.force,
    "verbose"          : opt.verbosity >= 1,
    "quiet"            : opt.quiet,
    "threads"          : opt.threads,
    "debug"            : opt.verbosity == 3,
    "exclude_patterns" : exclude_list,
    "new_engine"       : opt.new_engine,
    "def_engine"       : opt.def_engine,
    "all"              : opt.all,
    "locking"          : opt.locking,
    "use_regexp"       : opt.use_regexp,
    "rpl_user"         : opt.rpl_user,
    "rpl_mode"         : opt.rpl_mode,
    "verbosity"        : opt.verbosity,
    "skip_gtid"        : opt.skip_gtid,
}

# Parse source connection values
try:
    # Create a basic configuration reader first for optimization purposes.
    # I.e., to avoid repeating the execution of some methods in further
    # parse_connection methods (like, searching my_print_defaults tool).
    config_reader = MyDefaultsReader(options, False)
    source_values = parse_connection(opt.source, config_reader, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Source connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Source connection values invalid: %s." % err.errmsg)

# Parse destination connection values
try:
    dest_values = parse_connection(opt.destination, config_reader, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Destination connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Destination connection values invalid: %s."
                 % err.errmsg)

# Check to see if attempting to use --rpl on the same server
if (opt.rpl_mode or opt.rpl_user) and source_values == dest_values:
    parser.error("You cannot use the --rpl option for copying on the "
                 "same server.")

# Check replication options
check_rpl_options(parser, opt)

# Build list of databases to copy
db_list = []
for db in args:
    # Split the database names considering backtick quotes
    grp = re.match(r"(`(?:[^`]|``)+`|\w+)(?:(?:\:)(`(?:[^`]|``)+`|\w+))?", db)
    if not grp:
        parser.error(PARSE_ERR_DB_PAIR.format(db_pair=db,
                                              db1_label='orig_db',
                                              db2_label='new_db'))
    db_entry = grp.groups()
    orig_db, new_db = db_entry

    # Verify if the size of the databases matched by the REGEX is equal to the
    # initial specified string. In general, this identifies the missing use
    # of backticks.
    matched_size = len(orig_db)
    if new_db:
        # add 1 for the separator ':'
        matched_size = matched_size + 1
        matched_size = matched_size + len(new_db)
    if matched_size != len(db):
        parser.error(PARSE_ERR_DB_PAIR_EXT.format(db_pair=db,
                                                  db1_label='orig_db',
                                                  db2_label='new_db',
                                                  db1_value=orig_db,
                                                  db2_value=new_db))

    # Remove backtick quotes (handled later)
    orig_db = remove_backtick_quoting(orig_db) \
                if is_quoted_with_backticks(orig_db) else orig_db
    new_db = remove_backtick_quoting(new_db) \
                if new_db and is_quoted_with_backticks(new_db) else new_db
    db_entry = (orig_db, new_db)
    db_list.append(db_entry)

try:
    # record start time
    if opt.verbosity >= 3:
        start_test = time.time()
    dbcopy.copy_db(source_values, dest_values, db_list, options)
    if opt.verbosity >= 3:
        print_elapsed_time(start_test)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import re
import sys

from mysql.utilities.command.grep import ObjectGrep, OBJECT_TYPES
from mysql.utilities.common.options import add_regexp
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.tools import check_connector_python

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              "mysqlmetagrep - search metadata",
                              "%prog --server=user:pass@host:port:socket "
                              "[options] pattern", True)

# Setup utility-specific options:
parser.add_option("-b", "--body",
                  dest="check_body", action="store_true", default=False,
                  help="search the body of routines, triggers, and events as well")

def quote(string):
    return "'" + string + "'"

# Add some more advanced parsing here to handle types better.
parser.add_option(
    '--search-objects', '--object-types',
    dest="object_types", default=','.join(OBJECT_TYPES),
    help="the object type to search in: a comma-separated list"
    " of one or more of: " + ', '.join(map(quote, OBJECT_TYPES)))

# Add regexp
add_regexp(parser)

parser.add_option(
    "-p", "--print-sql", "--sql",
    dest="print_sql", action="store_true", default=False,
    help="print the statement instead of sending it to the server")
parser.add_option(
    "-e", "--pattern",
    dest="pattern",
    help="pattern to use when matching. Required if the pattern looks like a connection specification.")
parser.add_option(
    "--database",
    dest="database_pattern", default=None,
    help="only look at objects in databases matching this pattern")

# Output format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", "grid")

options, args = parser.parse_args()

_LOOKS_LIKE_CONNECTION_MSG = """Pattern '{pattern}' looks like a
connection specification. Use --pattern if this is really what you
want"""

_AT_LEAST_ONE_SERVER_MSG = """You need at least one server if you're
not using the --sql option"""

_NO_SERVERS_ALLOWED_MSG = """You should not include servers in the
call if you are using the --sql option"""

# A --pattern is required.
if not options.pattern:
    parser.error("No pattern supplied.")

# Check that --sql option is not used with --server, and --server are
# supplied if --sql is not used.
if options.print_sql:
    if options.server is not None and len(options.server) > 0:
        parser.error(_NO_SERVERS_ALLOWED_MSG)
else:
    if options.server is None or len(options.server) == 0:
        parser.error(_AT_LEAST_ONE_SERVER_MSG)

object_types = re.split(r"\s*,\s*", options.object_types)
command = ObjectGrep(options.pattern, options.database_pattern, object_types,
                     options.check_body, options.use_regexp)

try:
    if options.print_sql:
        print(command.sql())
    else:
        command.execute(options.server, format=options.format)
except Exception:
    _, details, _ = sys.exc_info()
    sys.stderr.write('ERROR: %s' % details)

sys.exit()



#
# Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
"""
This files contains unit tests for reading the MySQL login configuration file.
"""
import os
import subprocess
import tempfile
import time
import unittest

from mysql.utilities.exception import UtilError

from mysql.utilities.common.my_print_defaults import my_login_config_exists
from mysql.utilities.common.my_print_defaults import my_login_config_path
from mysql.utilities.common.my_print_defaults import MyDefaultsReader
from mysql.utilities.common.tools import get_tool_path

_TEST_LOGIN_PATH = 'test_my_print_defaults'
_TEST_USER = '127.0.0.1'
_TEST_HOST = 'localuser'
_TEST_UNKNOWN_LOGIN_PATH = 'test_unknown_group_data'


def mysql_tools_found():
    # mysql_config_editor needs to be accessible to run the tests
    try:
        return get_tool_path(None, "mysql_config_editor", search_PATH=True)
    except Exception:
        return None


@unittest.skipUnless(mysql_tools_found(),
                     "MySQL client tools (e.g. mysql_config_editor) must be "
                     "accessible to run this test.")
class TestMyPrintDefaults(unittest.TestCase):
    # NOTE: MySQL client tools (e.g., my_print_defaults) must be accessible,
    # for example by including their location in the PATH.

    @classmethod
    def setUpClass(cls):
        # Find mysql_config_editor to manipulate data from .mylogin.cnf
        try:
            cls.edit_tool_path = get_tool_path(None, "mysql_config_editor",
                                               search_PATH=True)
        except UtilError as err:
            raise UtilError("MySQL client tools must be accessible to run "
                            "this test (%s). Please add the location of the "
                            "MySQL client tools to your PATH." % err.errmsg)

        # Create login-path data
        cmd = ("{mysql_config_editor} set --login-path={login_path} "
               "--host={host} --user={user}")
        cmd = cmd.format(mysql_config_editor=cls.edit_tool_path,
                         login_path=_TEST_LOGIN_PATH,
                         host=_TEST_HOST, user=_TEST_USER)

        # Execute command to create login-path data
        proc = subprocess.Popen(cmd.split(' '), stdout=subprocess.PIPE,
                                stdin=subprocess.PIPE)
        # Overwrite login-path if already exists (i.e. answer 'y' to question)
        proc.communicate(input='y')

    @classmethod
    def tearDownClass(cls):
        # Remove login-path data
        cmd = "{mysql_config_editor} remove --login-path={login_path}"
        cmd = cmd.format(mysql_config_editor=cls.edit_tool_path,
                         login_path=_TEST_LOGIN_PATH)

        # Create a temporary file to redirect stdout
        out_file = tempfile.TemporaryFile()

        # Execute command to remove login-path data
        subprocess.call(cmd.split(' '), stdout=out_file)

    def setUp(self):
        # For safety, store the current PATH (to be restored)
        self.env_path = os.environ['PATH']

    def tearDown(self):
        # For safety, restore the PATH (previously stored)
        os.environ['PATH'] = self.env_path

    def test_my_login_config_path(self):
        # Check if the path (directory) to .mylogin.cnf exists
        path = my_login_config_path()
        self.assertTrue(os.path.isdir(path))

    def test_my_login_config_exists(self):
        # Check if .my_login.cnf file exists
        self.assertTrue(my_login_config_exists())

    def test_MyDefaultsReader(self):
        # Test the creation of MyDefaultsReader object

        # find_my_print_defaults_tool=False
        cfg_reader = MyDefaultsReader(find_my_print_defaults_tool=False)
        self.assertIsNone(cfg_reader.tool_path)
        self.assertDictEqual(cfg_reader._config_data, {})

        # find_my_print_defaults_tool=True (by default)
        cfg_reader = MyDefaultsReader()
        self.assertDictEqual(cfg_reader._config_data, {})
        self.assertTrue(os.path.isfile(cfg_reader.tool_path))

        # keep the location of the tool (to use as basedir)
        tool_dir = os.path.dirname(cfg_reader.tool_path)

        # Empty the PATH - UtilError raised if tool is not found
        os.environ['PATH'] = ''
        self.assertRaises(UtilError, MyDefaultsReader)

        # Specify a basedir to find the tool (PATH empty)
        options = {
            'basedir': tool_dir
        }
        cfg_reader = MyDefaultsReader(options)
        self.assertDictEqual(cfg_reader._config_data, {})
        self.assertTrue(os.path.isfile(cfg_reader.tool_path))

    def test_search_my_print_defaults_tool(self):
        # Test the search for the my_print_defaults tool
        cfg_reader = MyDefaultsReader(find_my_print_defaults_tool=False)
        self.assertIsNone(cfg_reader.tool_path)

        cfg_reader.search_my_print_defaults_tool()
        self.assertTrue(os.path.isfile(cfg_reader.tool_path))

        # keep the location of the tool (to use as search_path)
        tool_dir = os.path.dirname(cfg_reader.tool_path)

        # Empty the PATH - UtilError raised if tool is not found
        os.environ['PATH'] = ''
        cfg_reader = MyDefaultsReader(find_my_print_defaults_tool=False)
        self.assertIsNone(cfg_reader.tool_path)
        self.assertRaises(UtilError, cfg_reader.search_my_print_defaults_tool)

        # Specify the search paths to find the tool (PATH empty)
        self.assertRaises(UtilError, cfg_reader.search_my_print_defaults_tool,
                          ['not-a-valid-path'])
        cfg_reader.search_my_print_defaults_tool(['not-a-valid-path',
                                                  tool_dir])
        self.assertTrue(os.path.isfile(cfg_reader.tool_path))

    def test_check_tool_version(self):
        # Test the check of the my_print_defaults tool version
        # NOTE: current tool version is 1.6 (not have been update lately)
        cfg_reader = MyDefaultsReader()
        self.assertTrue(cfg_reader.check_tool_version(1, 6),
                        "Only version 1.6 or above of my_prints_defaults tool"
                        " is supported. Please add the location of a "
                        "supported tool version to your PATH (at the "
                        "beginning).")
        self.assertTrue(cfg_reader.check_tool_version(1, 0))
        self.assertFalse(cfg_reader.check_tool_version(1, 99))
        self.assertFalse(cfg_reader.check_tool_version(99, 0))
        self.assertTrue(cfg_reader.check_tool_version(0, 0))
        self.assertTrue(cfg_reader.check_tool_version(0, 99))

    def test_check_login_path_support(self):
        # Test if the my_print_defaults tool supports login-path options
        # NOTE: The version of the my_print_defaults tool as not been updated
        # properly (latest known version is 1.6). Therefore, not all the
        # my_print_defaults tools that report the version 1.6 support the use
        # of login-path.

        cfg_reader = MyDefaultsReader()
        self.assertTrue(cfg_reader.check_login_path_support(),
                        "The used my_prints_defaults tool does not support "
                        "login-path options: %s. Please add the location of a"
                        "tool that support login-path to your PATH (at the "
                        "beginning)." % cfg_reader.tool_path)

    def test_get_group_data(self):
        # Test the retrieve of the login-path group data from .mylogin.cnf

        cfg_reader = MyDefaultsReader(find_my_print_defaults_tool=False)

        # Test the assertion: First, my_print_defaults must be found.
        self.assertRaises(AssertionError, cfg_reader.get_group_data,
                          _TEST_LOGIN_PATH)

        cfg_reader.search_my_print_defaults_tool()

        expected_res = {
            'host': _TEST_HOST,
            'user': _TEST_USER
        }
        # 1rst time, get data from my_prints_defaults
        start_time1 = time.time()
        grp_data = cfg_reader.get_group_data(_TEST_LOGIN_PATH)
        end_time1 = time.time()
        self.assertEqual(expected_res, grp_data)

        # 2nd time, return previously obtained information (faster)
        start_time2 = time.time()
        grp_data = cfg_reader.get_group_data(_TEST_LOGIN_PATH)
        end_time2 = time.time()
        self.assertEqual(expected_res, grp_data)

        # Must be faster to retrieve the same data the second time
        self.assertLess((end_time2 - start_time2), (end_time1 - start_time1))

        # Return None if the specified group does not exist
        grp_data = cfg_reader.get_group_data(_TEST_UNKNOWN_LOGIN_PATH)
        self.assertIsNone(grp_data)

    def test_get_option_value(self):
        # Test the retrieve of specific option values from .mylogin.cnf

        cfg_reader = MyDefaultsReader(find_my_print_defaults_tool=False)

        # Test the assertion: First, my_print_defaults must be found.
        self.assertRaises(AssertionError, cfg_reader.get_option_value,
                          _TEST_LOGIN_PATH, _TEST_HOST)
        self.assertRaises(AssertionError, cfg_reader.get_option_value,
                          _TEST_LOGIN_PATH, _TEST_USER)

        cfg_reader.search_my_print_defaults_tool()

        expected_res = {
            'host': _TEST_HOST,
            'user': _TEST_USER
        }

        # Get user
        opt_value = cfg_reader.get_option_value(_TEST_LOGIN_PATH, 'user')
        self.assertEqual(expected_res['user'], opt_value)

        # Get host
        opt_value = cfg_reader.get_option_value(_TEST_LOGIN_PATH, 'host')
        self.assertEqual(expected_res['host'], opt_value)

        # Return None if the specified option/group does not exist
        opt_value = cfg_reader.get_option_value(_TEST_LOGIN_PATH, 'unknown')
        self.assertIsNone(opt_value)
        opt_value = cfg_reader.get_option_value(_TEST_UNKNOWN_LOGIN_PATH,
                                                'user')
        self.assertIsNone(opt_value)
        opt_value = cfg_reader.get_option_value(_TEST_UNKNOWN_LOGIN_PATH,
                                                'host')
        self.assertIsNone(opt_value)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the server information utility.
"""
from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import sys

from mysql.utilities.command.serverinfo import show_server_info
from mysql.utilities.common.options import add_basedir_option
from mysql.utilities.common.options import check_basedir_option
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.tools import check_connector_python

from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqlserverinfo "
DESCRIPTION = "mysqlserverinfo - show server information"
USAGE = "%prog --server=user:pass@host:port:socket --format=grid"

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, True)

# Setup utility-specific options:

# Input format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", "grid")

# Header row
parser.add_option("-h", "--no-headers", action="store_true", dest="no_headers",
                  default=False, help="do not show column headers")

# Show my.cnf values
parser.add_option("-d", "--show-defaults", action="store_true",
                  dest="show_defaults", default=False,
                  help="show defaults from the config file per server")

# Add --start option
parser.add_option("-s", "--start", action="store_true", dest="start",
                  help="start server in read only mode if offline")

# Add --basedir option
add_basedir_option(parser)

# Add --datadir option
parser.add_option("--datadir", action="store", dest="datadir", default=None,
                  type="string", help="the data directory for the server")

# Add --search-port
parser.add_option("--port-range", action="store", dest="ports",
                  default="3306:3333",
                  type="string", help="the port range to search for running"
                  " mysql servers on Windows systems")

# Add --show-servers option
parser.add_option("--show-servers", action="store_true", dest="show_servers",
                  help="show any known MySQL servers running on this host")

# Add startup timeout
parser.add_option("--start-timeout", action="store", dest="start_timeout",
                  type=int, default=10, help="Number of seconds to wait for "
                  "the server to start. Default = 10.")

# Add verbosity mode
add_verbosity(parser, False)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Check the basedir option for errors (e.g., invalid path)
check_basedir_option(parser, opt.basedir)

# Check start timeout for minimal value
if int(opt.start_timeout) < 10:
    opt.start_timeout = 10
    print("# WARNING: --start-timeout must be >= 10 seconds. Using "
          "default value 10.")

# Check port range
if os.name == 'nt':
    parts = opt.ports.split(":")
    if len(parts) != 2:
        print("# WARNING : {0} is not a valid port range. "
              "Using default." .format(opt.ports))
        opt.ports = "3306:3333"

# Set options for database operations.
options = {
    "format"        : opt.format,
    "no_headers"    : opt.no_headers,
    "verbosity"     : opt.verbosity,
    "debug"         : opt.verbosity >= 3,
    "show_defaults" : opt.show_defaults,
    "start"         : opt.start,
    "basedir"       : opt.basedir,
    "datadir"       : opt.datadir,
    "ports"         : opt.ports,
    "show_servers"  : opt.show_servers,
    "start_timeout" : opt.start_timeout,
}

if opt.server is None:
    parser.error("You must specify at least one server.")

try:
    show_server_info(opt.server, options)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)
except Exception:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e)
    sys.exit(1)

print("#...done.")

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the audit log administration utility which allows users to
manage the audit log (i.e., view/edit control variables; perform on-demand
log file rotation, and copy log files to other locations).
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.exception import UtilError, FormatError
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.options import CaseInsensitiveChoicesOption
from mysql.utilities.common.options import UtilitiesParser
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.common.tools import show_file_statistics
from mysql.utilities.command import audit_log
from mysql.utilities.command.audit_log import AuditLog
from mysql.utilities.command.audit_log import command_requires_value
from mysql.utilities.command.audit_log import command_requires_log_name
from mysql.utilities.command.audit_log import command_requires_server
from mysql.utilities import VERSION_FRM


class MyParser(UtilitiesParser):
    def format_epilog(self, formatter):
        return self.epilog

# Constants
NAME = "MySQL Utilities - mysqlauditadmin "
DESCRIPTION = "mysqlauditadmin - audit log maintenance utility "
USAGE = "%prog --server=user:pass@host:port --show-options "

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = MyParser(
    version=VERSION_FRM.format(program=os.path.basename(sys.argv[0])),
    description=DESCRIPTION,
    usage=USAGE,
    add_help_option=False,
    option_class=CaseInsensitiveChoicesOption,
    epilog=audit_log.VALID_COMMANDS_TEXT)

# Default option to provide help information
parser.add_option("--help", action="help", help="display this help message "
                  "and exit")

# Setup utility-specific options:

# Connection information for the source server
parser.add_option("--server", action="store", dest="server",
                  type="string", default=None,
                  help="connection information for the server in " + \
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Audit Log name (full path)
parser.add_option("--audit-log-name", action="store", dest="log_name",
                  type="string", default=None,
                  help="full path and file name for the audit log file. "
                  "Used for stats and copy options.")

# Show variables
parser.add_option("--show-options", action="store_true", dest="show_options",
                  help="display the audit log system variables.")

# Remote login
parser.add_option("--remote-login", action="store", dest="rlogin",
                  type="string", default=None,
                  help="user name and host to be used for remote login for "
                  "copying log files. Format: <user>:<host_or_ip> Password "
                  "will be prompted.")

# See file statistics
parser.add_option("--file-stats", action="store_true", default=False,
                  dest="file_stats",
                  help="display the audit log file statistics.")

# Copy file
parser.add_option("--copy-to", action="store", dest="copy_location",
                  type="string", default=None,
                  help="the location to copy the audit log file specified. "
                  "The path must be locally accessible for the current "
                  "user.")

# Value for command option
parser.add_option("--value", action="store", dest="value", default=None,
                  type="string", help="value used to set variables based "
                  "on the command specified. See --help for list per "
                  "command.")

# Add verbosity mode
add_verbosity(parser, False)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()


# Perform error checking

# One command at a time
if len(args) > 1:
    parser.error("You can only perform one command at a time.")

# Valid command?
if args and not args[0].upper() in audit_log.VALID_COMMANDS:
    parser.error("The command '%s' is not a valid command." % args[0])

if args:
    command = args[0].upper()
else:
    command = None

# At least one valid option must be specified
if (not opt.log_name and not opt.rlogin and not opt.value and not opt.server
    and not opt.copy_location and not opt.show_options
    and opt.file_stats == False):
    parser.error("At least one valid option must be specified.")

# if command, check to see if it requires a value.
if command and command_requires_value(command) and not opt.value:
    parser.error("The command %s requires the --value option." % command)

# The --value option must be used with a valid command
if opt.value and not command_requires_value(command):
    parser.error("The --value option must be used with a valid command.")

# The --server option is required.
if command_requires_server(command) and not opt.server:
    parser.error("The --server option is required for the %s command." %
                 command)

# The --server option must be used with --show-options and/or a valid command
if opt.server and (not opt.show_options
                   and not command_requires_server(command)):
    parser.error("The --server option requires --show-options and/or "
                 "a valid command.")

# The --server option is also required by --show-options
if opt.show_options and not opt.server:
    parser.error("The --server option is required for --show-options.")


# The --audit-log-name is required if a command specified.
if command_requires_log_name(command) and not opt.log_name:
    parser.error("The --audit-log-name option is required for the %s command."
                 % command)

if opt.log_name and (not opt.file_stats
                     and not command_requires_log_name(command)):
    parser.error("The --audit-log-name option requires --file-stats and/or "
                 "a valid command.")

# Attempt to parse the --server option
server_values = None
if opt.server:
    try:
        server_values = parse_connection(opt.server, None, opt)
    except FormatError:
        _, err, _ = sys.exc_info()
        parser.error("Server connection values invalid: %s." % err)
    except UtilError:
        _, err, _ = sys.exc_info()
        parser.error("Server connection values invalid: %s." % err.errmsg)

# Check for copy prerequisites
if command and command == "COPY" and not opt.copy_location:
    parser.error("You must specify the --copy-to option for copying a log "
                 "file.")

# The --copy-to option requires the command COPY
if opt.copy_location and not (command == "COPY"):
    parser.error("The --copy-to option can only be used with the COPY "
                 "command.")

# Check copy-to location
if (command and command == "COPY" and opt.copy_location) and \
   not os.access(opt.copy_location, os.W_OK | os.R_OK):
    parser.error("You must have read and write access to the destination "
                 "for audit log copy.")

# Check args for copy-to, file-stats
if ((command and command == "COPY" and opt.copy_location) or
    opt.file_stats) and not opt.log_name:
    parser.error("You must specify the --audit-log-name option for copying "
                 "log files or viewing file statistics.")

# Check if the specified audit-log-name is a file
if opt.log_name and not opt.rlogin and not os.path.isfile(opt.log_name):
    parser.error("The specified --audit-log-name is not a file: %s" \
                  % opt.log_name)

# Check remote login format
if opt.rlogin:
    try:
        user, host = opt.rlogin.split(":", 1)
    except:
        parser.error("The --remote-login option should be in the format: "
                     "<user>:<host_or_ip>")

    if not (command and  command == "COPY"):
        parser.error("The --remote-login option can only be used with the COPY "
                     "command.")


# Create dictionary of options
options = {
    'verbosity'     : opt.verbosity,
    'command'       : command,
    'log_name'      : opt.log_name,
    'server_vals'   : server_values,
    'rlogin'        : opt.rlogin,
    'file_stats'    : opt.file_stats,
    'show_options'  : opt.show_options,
    'copy_location' : opt.copy_location,
    'value'         : opt.value,
}

try:
    # Open a connection to the audit log manager and run the audit
    # log commands as specified by the options.
    log = AuditLog(options)

    # Make sure server supports the audit log else fail
    if command != "COPY" and not opt.file_stats:
        log_error = log.check_audit_log()
        if log_error:
            parser.error(log_error)

    # Show audit log options before command
    if opt.show_options:
        # if some other command has run
        if len(args):
            print("#\n# Showing options before command.")
        log.show_options()

    # Execute the command specified
    if len(args):
        log.do_command()

    # Show audit log options after command if appropriate
    if opt.show_options and len(args):
        # if some other command has run
        print("#\n# Showing options after command.")
        log.show_options()

    # Do file stats
    if opt.file_stats:
        show_file_statistics(opt.log_name, True)

except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the disk usage utility for showing the estimated disk
storage of the databases and system files.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import sys
import time

from mysql.utilities.command import diskusage
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.options import add_format_option
from mysql.utilities.common.server import connect_servers
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqldiskusage "
DESCRIPTION = "mysqldiskusage - show disk usage for databases"
USAGE = "%prog --server=user:pass@host:port:socket db1 --all"

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

def print_elapsed_time(start_test):
    """Print the elapsed time to stdout (screen)

    start_test[in]      The starting time of the test
    """
    stop_test = time.time()
    display_time = int((stop_test - start_test) * 100)
    if display_time == 0:
        display_time = 1
    print("Time: %6d\n" % display_time)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE)

# Setup utility-specific options:

# Output format
add_format_option(parser, "display the output in either grid (default), "
                  "tab, csv, or vertical format", "grid")

# Header row
parser.add_option("-h", "--no-headers", action="store_true", dest="no_headers",
                  default=False, help="do not show column headers")

# Binlogs option
parser.add_option("-b", "--binlog", action="store_true", dest="do_binlog",
                  default=False, help="include binary log usage")

# Relay logs option
parser.add_option("-r", "--relaylog", action="store_true", dest="do_relaylog",
                  default=False, help="include relay log usage")

# Logs option
parser.add_option("-l", "--logs", action="store_true", dest="do_logs",
                  default=False, help="include general and slow log usage")

# Innodb option
parser.add_option("-i", "--innodb", action="store_true", dest="do_innodb",
                  default=False, help="include InnoDB tablespace usage")

# Show empty databases option
parser.add_option("-m", "--empty", action="store_true", dest="do_empty",
                  default=False, help="include empty databases")

# all option
parser.add_option("-a", "--all", action="store_true", dest="do_all",
                  default=False, help="show all usage including empty "
                  "databases")


# Add verbosity mode
add_verbosity(parser, True)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Parse source connection values
try:
    source_values = parse_connection(opt.server, None, opt)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Source connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Source connection values invalid: %s." % err.errmsg)

try:
    conn_options = {
        'version'   : "5.1.30",
    }
    servers = connect_servers(source_values, None)
except UtilError:
    _, e, _ = sys.exc_info()
    parser.error(e.errmsg)

try:
    res = servers[0].show_server_variable("datadir")
    datadir = res[0][1]
except UtilError:
    _, e, _ = sys.exc_info()
    parser.error(e.errmsg)

if not os.access(datadir, os.R_OK):
    print("\nNOTICE: Your user account does not have read access to the "
          "datadir. Data sizes will be calculated and actual file sizes "
          "may be omitted. Some features may be unavailable.\n")

# Set options for database operations.
options = {
    "format"        : opt.format,
    "no_headers"    : opt.no_headers,
    "verbosity"     : opt.verbosity,
    "debug"         : opt.verbosity >= 3,
    "have_read"     : os.access(datadir, os.R_OK),
    "do_empty"      : opt.do_empty,
    "do_all"        : opt.do_all,
    "quiet"         : opt.quiet
}

# We do database disk usage by default.
try:
    diskusage.show_database_usage(servers[0], datadir, args, options)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Look for the general and query logs and report
if opt.do_logs or opt.do_all:
    try:
        diskusage.show_logfile_usage(servers[0], options)
    except UtilError:
        _, e, _ = sys.exc_info()
        print("ERROR: %s" % e.errmsg)
        sys.exit(1)

# Look for the binary logs and report
if opt.do_binlog or opt.do_all:
    try:
        options["log_type"] = 'binary log'
        diskusage.show_log_usage(servers[0], datadir, options)
    except UtilError:
        _, e, _ = sys.exc_info()
        print("ERROR: %s" % e.errmsg)
        sys.exit(1)

# Look for the relay logs and report
if opt.do_relaylog or opt.do_all:
    try:
        options["log_type"] = 'relay log'
        diskusage.show_log_usage(servers[0], datadir, options)
    except UtilError:
        _, e, _ = sys.exc_info()
        print("ERROR: %s" % e.errmsg)
        sys.exit(1)

# Look at the inoodb tablespace information are report
if opt.do_innodb or opt.do_all:
    try:
        diskusage.show_innodb_usage(servers[0], datadir, options)
    except UtilError:
        _, e, _ = sys.exc_info()
        print("ERROR: %s" % e.errmsg)
        sys.exit(1)

if not opt.quiet:
    print("#...done.")

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the replicate utility. It is used to establish a
master/slave replication topology among two servers.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os.path
import sys

from mysql.utilities.exception import UtilError
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.messages import (PARSE_ERR_OPTS_REQ,
                                             WARN_OPT_USING_DEFAULT)
from mysql.utilities.common.options import add_verbosity
from mysql.utilities.common.server import check_hostname_alias
from mysql.utilities.common.tools import check_connector_python
from mysql.utilities.command.check_rpl import check_replication
from mysql.utilities.exception import FormatError

# Constants
NAME = "MySQL Utilities - mysqlrplcheck "
DESCRIPTION = "mysqlrplcheck - check replication"
USAGE = "%prog --master=root@localhost:3306 --slave=root@localhost:3310 "

PRINT_WIDTH = 75

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

# Setup the command parser
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE, True, False)

# Setup utility-specific options:

# Connection information for the source server
parser.add_option('--master', action="store", dest="master",
                  type="string", default=None,
                  help="connection information for master server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>].")

# Connection information for the destination server
parser.add_option('--slave', action="store", dest="slave",
                  type="string", default=None,
                  help="connection information for slave server in "
                  "the form: <user>[:<password>]@<host>[:<port>][:<socket>]"
                  " or <login-path>[:<port>][:<socket>]")

# Add --master-info-file
parser.add_option("--master-info-file", action="store", dest="master_info",
                  type="string", default="master.info",
                  help="the name of the master information file on the slave."
                       "default = 'master.info' read from the data directory."
                       " Note: this option requires that the utility run on "
                       "the slave with appropriate file read access to the "
                       "data directory.")

# Add --show-slave-status
parser.add_option("--show-slave-status", "-s", action="store_true",
                  dest="slave_status", default=False, help="show slave status")

# Add display width option
parser.add_option("--width", action="store", dest="width",
                  type="int", help="display width",
                  default=PRINT_WIDTH)

# Add suppress to suppress warning messages
parser.add_option("--suppress", action="store_true", dest="suppress",
                  default=False, help="suppress warning messages")

# Add verbosity
add_verbosity(parser, True)

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# option --master is required (mandatory)
if not opt.master:
    default_val = 'root@localhost:3306'
    print(WARN_OPT_USING_DEFAULT.format(default=default_val, opt='--master'))
    # Print the WARNING to force determinism if a parser error occurs.
    sys.stdout.flush()

# option --slave is required (mandatory)
if not opt.slave:
    parser.error(PARSE_ERR_OPTS_REQ.format(opt='--slave'))

# Parse source connection values
try:
    m_values = parse_connection(opt.master)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Master connection values invalid: %s." % err.errmsg)

# Parse source connection values
try:
    s_values = parse_connection(opt.slave)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Slave connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Slave connection values invalid: %s." % err.errmsg)

# Check hostname alias
if check_hostname_alias(m_values, s_values):
    parser.error("The master and slave are the same host and port.")

# Create dictionary of options
options = {
    'verbosity'    : opt.verbosity,
    'pedantic'     : False,
    'quiet'        : opt.quiet,
    'suppress'     : opt.suppress,
    'master_info'  : opt.master_info,
    'slave_status' : opt.slave_status,
    'width'        : opt.width
}

try:
    res = check_replication(m_values, s_values, options)
    if res:
        sys.exit(1)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()



#!/usr/bin/env python
#
# Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

"""
This file contains the export database utility which allows users to export
metadata for objects in a database and data for tables.
"""

from mysql.utilities.common.tools import check_python_version

# Check Python version compatibility
check_python_version()

import os
import sys
import time
from mysql.utilities.command.dbexport import export_databases
from mysql.utilities.common.ip_parser import parse_connection
from mysql.utilities.common.options import add_regexp
from mysql.utilities.common.options import setup_common_options
from mysql.utilities.common.options import add_skip_options, check_skip_options
from mysql.utilities.common.options import add_verbosity, check_verbosity
from mysql.utilities.common.options import add_format_option, add_rpl_mode
from mysql.utilities.common.options import add_all, check_all, add_locking
from mysql.utilities.common.options import add_rpl_user, check_rpl_options
from mysql.utilities.common.sql_transform import remove_backtick_quoting
from mysql.utilities.common.sql_transform import is_quoted_with_backticks
from mysql.utilities.common.tools import check_connector_python

from mysql.utilities.exception import FormatError
from mysql.utilities.exception import UtilError

# Constants
NAME = "MySQL Utilities - mysqldbexport "
DESCRIPTION = "mysqldbexport - export metadata and data from databases"
USAGE = "%prog --server=user:pass@host:port:socket db1, db2, db3"

_PERMITTED_DISPLAY = ["names", "brief", "full"]
_PERMITTED_EXPORTS = ["data", "definitions", "both"]

# Check for connector/python
if not check_connector_python():
    sys.exit(1)

def print_elapsed_time(start_test):
    """Print the elapsed time to stdout (screen)

    start_test[in]      The starting time of the test
    """
    stop_test = time.time()
    display_time = int((stop_test - start_test) * 100)
    if display_time == 0:
        display_time = 1
    print("Time: %6d\n" % display_time)

# Setup the command parser and setup server, help
parser = setup_common_options(os.path.basename(sys.argv[0]),
                              DESCRIPTION, USAGE)

# Setup utility-specific options:

# Output format
add_format_option(parser, "display the output in either sql (default), "
                  "grid, tab, csv, or vertical format", "sql", True)

# Display format
parser.add_option("-d", "--display", action="store", dest="display",
                  default="brief", help="control the number of columns shown: "
                  "'brief' = minimal columns for object creation (default), "
                  "'full' = all columns, 'names' = only object names (not "
                  "valid for --format=sql)", type="choice",
                  choices=_PERMITTED_DISPLAY)

# Export mode
parser.add_option("-e", "--export", action="store", dest="export",
                  default="definitions", help="control the export of either "
                  "'data' = only the table data for the tables in the database "
                  "list, 'definitions' = export only the definitions for "
                  "the objects in the database list, or 'both' = export "
                  "the metadata followed by the data "
                  "(default: export definitions)", type="choice",
                  choices=_PERMITTED_EXPORTS)

# Single insert mode
parser.add_option("-b", "--bulk-insert", action="store_true",
                  dest="bulk_import", default=False, help="use bulk insert "
                  "statements for data (default:False)")

# Header row
parser.add_option("-h", "--no-headers", action="store_true", dest="no_headers",
                  default=False, help="do not display the column headers - "
                  "ignored for grid format")

# Skip blobs for export
parser.add_option("--skip-blobs", action="store_true", dest="skip_blobs",
                  default=False, help="do not export blob data.")

# File-per-table mode
parser.add_option("--file-per-table", action="store_true", dest="file_per_tbl",
                  default=False, help="write table data to separate files. "
                  "Valid only for --export=data or --export=both.")

# Add the exclude database option
parser.add_option("-x", "--exclude", action="append", dest="exclude",
                  type="string", default=None, help="exclude one or more "
                  "objects from the operation using either a specific name "
                  "(e.g. db1.t1), a LIKE pattern (e.g. db1.t% or db%.%) or a "
                  "REGEXP search pattern. To use a REGEXP search pattern for "
                  "all exclusions, you must also specify the --regexp option. "
                  "Repeat the --exclude option for multiple exclusions.")

# Add the all database options
add_all(parser, "databases")

# Add the skip common options
add_skip_options(parser)

# Add verbosity and quiet (silent) mode
add_verbosity(parser, True)

# Add regexp
add_regexp(parser)

# Add locking
add_locking(parser)

# Replication user and password
add_rpl_user(parser, None)

# Add replication options
add_rpl_mode(parser)

parser.add_option("--skip-gtid", action="store_true", default=False,
                  dest="skip_gtid", help="skip creation of GTID_PURGED "
                  "statements.")

# Add comment replication output
parser.add_option("--comment-rpl", action="store_true", default=False,
                  dest="comment_rpl", help="place the replication statements "
                  "in comment statements. Valid only with --rpl option.")

parser.add_option("--skip-fkey-checks", action="store_true", default=False,
                  dest="skip_fkeys", help="skip creation of foreign key "
                  "disable/enable statements.")

# Now we process the rest of the arguments.
opt, args = parser.parse_args()

# Warn if quiet and verbosity are both specified
check_verbosity(opt)

try:
    skips = check_skip_options(opt.skip_objects)
except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

# Fail if no db arguments or all
if len(args) == 0 and not opt.all:
    parser.error("You must specify at least one database to export or "
                 "use the --all option to export all databases.")

# Check replication options
check_rpl_options(parser, opt)

# Fail if we have arguments and all databases option listed.
check_all(parser, opt, args, "databases")

if opt.skip_blobs and not opt.export == "data":
    print("# WARNING: --skip-blobs option ignored for metadata export.")

if opt.file_per_tbl and opt.export in ("definitions", "both"):
    print("# WARNING: --file-per-table option ignored for metadata export.")

if "data" in skips and opt.export == "data":
    print("ERROR: You cannot use --export=data and --skip-data when exporting "
          "table data.")
    sys.exit(1)

# Process --exclude values to remove unnecessary quotes (when used) in order
# to avoid further matching issues.
if opt.exclude:
    # Remove unnecessary outer quotes.
    exclude_list = [pattern.strip("'\"") for pattern in opt.exclude]
else:
    exclude_list = opt.exclude

# Set options for database operations.
options = {
    "skip_tables"      : "tables" in skips,
    "skip_views"       : "views" in skips,
    "skip_triggers"    : "triggers" in skips,
    "skip_procs"       : "procedures" in skips,
    "skip_funcs"       : "functions" in skips,
    "skip_events"      : "events" in skips,
    "skip_grants"      : "grants" in skips,
    "skip_create"      : "create_db" in skips,
    "skip_data"        : "data" in skips,
    "skip_blobs"       : opt.skip_blobs,
    "skip_fkeys"       : opt.skip_fkeys,
    "format"           : opt.format,
    "no_headers"       : opt.no_headers,
    "display"          : opt.display,
    "single"           : not opt.bulk_import,
    "quiet"            : opt.quiet,
    "verbosity"        : opt.verbosity,
    "debug"            : opt.verbosity >= 3,
    "file_per_tbl"     : opt.file_per_tbl,
    "exclude_patterns" : exclude_list,
    "all"              : opt.all,
    "use_regexp"       : opt.use_regexp,
    "locking"          : opt.locking,
    "rpl_user"         : opt.rpl_user,
    "rpl_mode"         : opt.rpl_mode,
    "rpl_file"         : opt.rpl_file,
    "comment_rpl"      : opt.comment_rpl,
    "export"           : opt.export,
    "skip_gtid"        : opt.skip_gtid,
}

# Parse server connection values
try:
    server_values = parse_connection(opt.server, None, options)
except FormatError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err)
except UtilError:
    _, err, _ = sys.exc_info()
    parser.error("Server connection values invalid: %s." % err.errmsg)

# Build list of databases to copy
db_list = []
for db in args:
    # Remove backtick quotes (handled later)
    db = remove_backtick_quoting(db) if is_quoted_with_backticks(db) else db
    db_list.append(db)

try:
    # record start time
    if opt.verbosity >= 3:
        start_test = time.time()

    # Export all databases specified
    export_databases(server_values, db_list, options)

    # record elapsed time
    if opt.verbosity >= 3:
        print_elapsed_time(start_test)

except UtilError:
    _, e, _ = sys.exc_info()
    print("ERROR: %s" % e.errmsg)
    sys.exit(1)

sys.exit()
